require 'nokogiri'

module Jaspion
  module Miya
    class Document < Nokogiri::XML::SAX::Document
      FLAT = 0
      TREE = 1

      # Array with Miya::Object objects
      attr_accessor :objects

      # Indicates if the elements should be a flat array or hierarchy
      attr_accessor :structure

      # Initializes a Document object and parses the xml argument if present
      def initialize(xml = nil, structure = TREE)
        @objects = []
        @structure = structure

        return if xml.nil?
        parse(xml)
      end

      # Replaces the current @objects variable
      def parse(xml)
        @objects = []
        parser = Nokogiri::XML::SAX::Parser.new(self)
        parser.parse(xml)
        raise 'Document cannot have more than one root element' if @objects.length > 1
      end

      # Nokogiri Document method
      def start_element_namespace(name, attrs = [], prefix = nil, uri = nil, ns = [])
      end

      # Nokogiri Document method
      def end_element_namespace(name, prefix = nil, uri = nil)
        return if name.nil?
        return if @structure == FLAT
        return if @objects.length <= 1

        last = @objects.delete(@objects.last)
        @objects.last.push_child(last)
      end
    end
  end
end
