# Author:: Toshiro Sugii
# Copyright:: Copyright (c) 2016 Toshiro Sugii
# License::   Distributes under the same terms as Ruby
require 'jaspion/miya/template'
require 'jaspion/miya/class'

module Jaspion
  module Miya
    # Represents a single class object
    class Object
      include Jaspion::Miya::Class

      # Object name - Like variable name
      attr_accessor :name

      # Object type - Like variable type
      attr_accessor :type

      # Objects can contain other object inside
      attr_accessor :children

      # Initializes an Object object
      # Object's type is defined by the last path of the Ruby class name
      #
      # @param name [String] Object name
      #
      # @param import [String] Import value
      def initialize(name = nil)
        @type = self.class.name.split('::').last
        @name = name
        @children = []
      end

      # Pushes an new Object object
      #
      # @param name [Object] Object object
      def push_child(object)
        children.push(object)
      end

      # Look for a Object type inside @objects array
      #
      # @param type [String] Name of type object to find
      #
      # @return [Object] If has object, returns it else returns nil
      def fetch_child(type)
        children.find { |c| c.type.eql?(type) }
      end

      # Generate an array of Template objects
      #
      # @return [Array] List of the Template object that belongs to this class
      def templates
        [Miya::Template.new(self, nil)]
      end

      # Creates a new instance of the specified Object depending on the
      # plataform's name
      #
      # @param class_name [String] Object class name
      # @param instance_name [String] Object's instance name
      #
      # @return [Object] The Object instance depending on the specified
      #                  plataform
      def self.create(class_name, instance_name)
        return nil unless const_defined?(class_name)

        clazz = const_get(class_name)
        clazz.new(instance_name)
      end

      # Returns all available template classes
      def self.available_classes
        descendants
      end
    end
  end
end
